#pragma once

#include "DialogManager.h"
#include "ChatModel.h"
#include "CommonType.h"

class CChatUser;
class CreateChatDialog;
class CChatDlg;

typedef enum
{
	MAIN_AREA, 
	MAIN_RIGHT_AREA,
	MAIN_LEFT_AREA
} AutoAlignIndex;

class ChatDialogManager : public DialogManager
{
public:
	ChatDialogManager(void);
	~ChatDialogManager(void);

	//void CreateBuddyChat(std::map<WSTRING, BYTE> &m_mapBuddyInfo);
	void CreateBuddyChat(const std::vector<const IGTBuddyUser*>& buddyVector, BUDDY_SENDACTIONType followAction=SENDACTION_NONE);	
	void CreateMemberChat(const std::vector<NE_MEMBER_DATA>& memberVector);
	void CreateSingleMemberChat(const CString& strUserID, const CString& strUserName); // for RecvMemoDlg
	void CreateSingleBuddyChat(const IGTBuddyUser* pUser);
	void CreateInvitedChat(CChatUser *pChatUser, const CString& serverAddress, int nPort, int nRoomNo, int nRoomType);
	CChatDlg* CreateJoinedChat(NE_MYROOM_DATA& myRoomData);
	void AddRecentChatRoomData(const NE_MYROOM_DATA& myRoomData);
	void RequestAllRecentUserStatus();
	NE_MYROOM_DATA* GetRoomData(int nRoomID);
	void CreateUserIDArrayChat(const CStringArray& userIDArray);  // for COPYDATA_CHAT
	void InitChatRoomList();
	BOOL ActivateChatDialog(int nRoomNo, BOOL bResize = FALSE);
	void SetForegroundChatDialog(int nRoomNo);
	BOOL DisplaySearchChatRoomList( CString strSearchText );
	void CloseChatDialogs(const CUIntArray& nRoomIDArray);
//	void CloseAllChatDialog();
    void CloseChatDialog(int nRoomID);
	void SetSessionRecoveryOfAllChatDialog( BOOL bTry );
	void InitializeScreenRects(BOOL bInitialize = TRUE);
	CPoint GetInsertScreenPoint(CChatDlg *pDlg);
	void InitializeSearchRect();
	CChatDlg* FindCreatedSessionChatDlg(int nRoomNo);
	void SetViewLanguagePack();
	void InvalidateChatDlg();
	CChatDlg* FindChatDlgByHWND(HWND hWnd);
	virtual void Remove(CDialog* pDialog, bool bRemoveRecentItem = false);
	virtual int RemoveAll();

	void UpdateBuddyStatus(LPCWSTR strBuddyID, int nTypeBits);
	void UpdateNotBuddyStatus(LPCWSTR strBuddyID, int nUserStatus, LPCWSTR strNick, int nIPTStatus, int nFlag);
	void UpdateMyStatus(int nFlag, int nUserStatus, LPCWSTR strNick, int nIPTStatus);
	void RemoveBuddy(LPCWSTR strBuddyID);
	CChatDlg* GetChatDialog(int nRoomNo);
	//static int __cdecl ItemComareFunc(void *pvlocale,const void *elem1, const void *elem2);
	int GetCountOfChatDlgNotClosed() /* when a chat dialog has file-open modal dialog there, it can't be closed. */;


private:
	map<int, NE_MYROOM_DATA> m_mapRecentChatRoomData;
	CCriticalSection m_csFindCreatedDlg;
	CRect m_chatAlignRects[3];	// 0:޽ â, 1:â , 2:â 
	CRect m_screenRect;
	int m_offset;
	
	bool IsAvailableWidth(CRect rtDlg);
	virtual void SetReadOnly(BOOL bStatus, CObject* pObject);
	CChatDlg* Create(CreateChatDialog *pCreateChatDialog, bool bSetFocus =false);
	int MakeSequenceID();
	void ReUserNickname(CChatDlg* pDlg, LPCTSTR strID, LPCTSTR strNickname);
	bool IsAvailableChatRoom();
	bool IsCreatedSingleChatRoom(const std::wstring& userId);
	CChatDlg* CreateChatDlg(const std::vector<CChatUser*>& userList, EChatMakeType eMakeType, int channel=0);
	void AddUserListChatDlg(CChatDlg* pChatDlg, std::vector<CChatUser*>& userList);
	CChatDlg* FindSingleChatDlg(LPCTSTR userId, int nRoomNo = -1);	//   1:1 ȭ ִ ã´.
	CChatDlg* FindCreatedSessionChatDlg(const std::vector<CChatUser *>& userList, LPCTSTR serverAddress, int nPort, int nRoomNo);
	bool CheckChatUserNumber(std::vector<CChatUser *> &vecUser);

	BOOL IsInside( CRect rectRegion, CRect rectTarget );
	void InitializeScreenRect(int index);
	CChatDlg* CreateAbnormalTerminatedChatDialog( CChatDlg* pChatDlg, std::vector<CChatUser *> &userVector, CSingleLock &removeLock, NE_MYROOM_DATA &myRoomData, /*LPCWSTR myUserID,*/ CString strLastMsgUserName, CString strLastMsgUserTalkName );
	BOOL IsHalf();
	bool CreateAbnormalChatItem(CChatDlg* pChatDlg, CString strLastMsgUserName, CString strLastMsgUserTalkName, NE_MYROOM_DATA &myRoomData);


};

class CreateChatDialog
{
public:
	CreateChatDialog()
	{

	}
	virtual  ~CreateChatDialog()
	{

	}
	virtual bool IsSingleChat() = 0;
	virtual const std::wstring GetUserID() = 0;
	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector) = 0;
};

class CreateBuddyChatDialog : public CreateChatDialog
{
public:
	CreateBuddyChatDialog(const std::vector<const IGTBuddyUser*>& buddyVector)
		: m_buddyVector(buddyVector)
	{

	}

	virtual bool IsSingleChat()
	{
		return (m_buddyVector.size() == 1);
	}

	virtual const std::wstring GetUserID()
	{
		std::vector<const IGTBuddyUser*>::const_iterator buddyIterator = m_buddyVector.begin();

		if (buddyIterator == m_buddyVector.end())
			return L"";

		const IGTBuddyUser* pUser = *buddyIterator;

		return std::wstring(pUser->GetUserID());
	}

	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector);

private:

	const std::vector<const IGTBuddyUser*>& m_buddyVector;
};

class CreateMemberChatDialog : public CreateChatDialog
{
public:
	CreateMemberChatDialog(const std::vector<NE_MEMBER_DATA>& memberVector)
		: m_memberVector(memberVector)
	{

	}

	virtual bool IsSingleChat()
	{
		return (m_memberVector.size() == 1);
	}

	virtual const std::wstring GetUserID()
	{
		std::vector<NE_MEMBER_DATA>::const_iterator memberIterator = m_memberVector.begin();

		if (memberIterator == m_memberVector.end())
			return L"";

		return memberIterator->strUserId;
	}

	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector);

private:
	const std::vector<NE_MEMBER_DATA>& m_memberVector;
};

class CreateSingleMemberChatDialog : public CreateChatDialog
{
public:
	CreateSingleMemberChatDialog(const CString& strUserID, const CString& strUserName)
		: m_strUserID(strUserID)
		, m_strUserName(strUserName)
	{

	}

	virtual bool IsSingleChat()
	{
		return true;
	}

	virtual const std::wstring GetUserID()
	{
		return std::wstring(m_strUserID);   //lint !e119
	}

	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector);

private:
	const CString& m_strUserID;
	const CString& m_strUserName;
};

class CreateSingleBuddyChatDialog : public CreateChatDialog
{
public:
	CreateSingleBuddyChatDialog(const IGTBuddyUser* pUser)
		: m_pUser(pUser)
	{
	}

	virtual bool IsSingleChat()
	{
		return true;
	}

	virtual const std::wstring GetUserID()
	{
		return std::wstring(m_pUser->GetUserID());
	}

	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector);

private:
	const IGTBuddyUser* m_pUser;
};

class CreateUserIDArrayChatDialog : public CreateChatDialog
{
public:
	CreateUserIDArrayChatDialog(const CStringArray& userIDArray)
		: m_userIDArray(userIDArray)
	{
	}

	virtual bool IsSingleChat()
	{
		return (m_userIDArray.GetCount() == 1);
	}

	virtual const std::wstring GetUserID()
	{
		return std::wstring(m_userIDArray[0]);  //lint !e119
	}

	virtual void MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector);

private:
	const CStringArray& m_userIDArray;
};